=== modified file 'NEWS'
--- NEWS	2007-01-24 19:42:26 +0000
+++ NEWS	2007-02-12 14:02:01 +0000
@@ -19,6 +19,12 @@
     * ``bzr help global-options`` describes the global options.
       (Aaron Bentley)
 
+    * HTTP redirections are now taken into account when a branch is accessed
+      for the first time. In the past, http redirections were silently
+      followed for each request which signifanctly degrade performances. The
+      http redirections are not followed anymore by default.
+      (Vincent Ladeuil)
+
   INTERNALS:
 
     * Reserved ids (any revision-id ending in a colon) are rejected by

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2007-02-11 16:06:13 +0000
+++ bzrlib/bzrdir.py	2007-02-12 14:02:01 +0000
@@ -523,7 +523,6 @@
                 format = BzrDirFormat.find_format(transport)
                 redirected = False
             except errors.RedirectRequested, e:
-
                 qualified_source = e.get_source_url()
                 relpath = transport.relpath(qualified_source)
                 if not e.target.endswith(relpath):
@@ -535,7 +534,13 @@
                      transport.base,
                      e.permanently,
                      target)
-                # Let's try with a new transport
+                # Let's try with a new transport 
+
+                # FIXME: If 'transport' have a qualifier, this
+                # should be applied again to the new transport
+                # *iff* the schemes used are the same. It's a bit
+                # tricky to verify, so I'll punt for now 
+                # -- vila20070212
                 qualified_target = e.get_target_url()[:-len(relpath)]
                 transport = get_transport(target)
                 redirections += 1
@@ -543,7 +548,10 @@
             # Out of the loop and still redirected ? Either the
             # user have kept a very very very old reference to a
             # branch or a loop occured in the redirections.
-            # Nothing we can cure here: tell the user.
+            # Nothing we can cure here: tell the user. Note that
+            # as the user have been informed about each
+            # redirection, there is no need to issue an
+            # additional error message.
             raise errors.NotBranchError(path=initial_base)
         BzrDir._check_supported(format, _unsupported)
         return format.open(transport, _found=True)
@@ -1507,23 +1515,6 @@
         """Allow changint the repository format for metadir formats."""
         self._repository_format = value
 
-    @classmethod
-    def probe_transport(klass, transport):
-        """Return the .bzrdir style transport present at URL.
-
-        Redirections are not followed.
-        """
-        try:
-            format_file = transport.get(".bzr/branch-format")
-            format_string = format_file.read()
-        except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base)
-
-        try:
-            return klass._formats[format_string]
-        except KeyError:
-            raise errors.UnknownFormatError(format=format_string)
-
     repository_format = property(__return_repository_format, __set_repository_format)
 
 

=== modified file 'bzrlib/tests/HTTPTestUtil.py'
--- bzrlib/tests/HTTPTestUtil.py	2007-02-04 17:41:12 +0000
+++ bzrlib/tests/HTTPTestUtil.py	2007-02-12 14:02:01 +0000
@@ -232,24 +232,30 @@
         """Redirect a single HTTP request to another host"""
         valid = TestingHTTPRequestHandler.parse_request(self)
         if valid:
-            # Just redirect permanently
-            self.send_response(301)
-            target = 'http://%s:%s%s' % (self.server.test_case.redirect_to_host,
-                                         self.server.test_case.redirect_to_port,
-                                         self.path)
-            self.send_header('Location', target)
-            self.end_headers()
-            return False # The job is done
+            rhost, rport = self.server.test_case.redirected_to_address()
+            if rhost is not None and rport is not None:
+                # Just redirect permanently
+                self.send_response(301)
+                target = 'http://%s:%s%s' % (rhost, rport, self.path)
+                self.send_header('Location', target)
+                self.end_headers()
+                return False # The job is done
         return valid
 
 
 class HTTPServerRedirecting(HttpServer):
     """An HttpServer redirecting to another server """
 
-    def __init__(self, redir_host, redir_port,
-                 request_handler=RedirectRequestHandler):
+    def __init__(self, request_handler=RedirectRequestHandler):
         HttpServer.__init__(self, request_handler)
-        self.redirect_to_host = redir_host
-        self.redirect_to_port = redir_port
+        self._redirect_to_host = None
+        self._redirect_to_port = None
+
+    def redirect_to(self, redir_host, redir_port):
+        self._redirect_to_host = redir_host
+        self._redirect_to_port = redir_port
+
+    def redirected_to_address(self):
+        return self._redirect_to_host, self._redirect_to_port
 
 

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2007-01-17 14:36:49 +0000
+++ bzrlib/tests/test_bzrdir.py	2007-02-12 14:02:01 +0000
@@ -35,9 +35,19 @@
                            UnsupportedFormatError,
                            )
 import bzrlib.repository as repository
-from bzrlib.tests import TestCase, TestCaseWithTransport, test_sftp_transport
+from bzrlib.tests import (
+    TestCase,
+    TestCaseWithTransport,
+    test_sftp_transport
+    )
 from bzrlib.tests.HttpServer import HttpServer
+from bzrlib.tests.HTTPTestUtil import (
+    TestCaseWithTwoWebservers,
+    HTTPServerRedirecting,
+    )
+from bzrlib.tests.test_http import TestWithTransport_pycurl
 from bzrlib.transport import get_transport
+from bzrlib.transport.http._urllib import HttpTransport_urllib
 from bzrlib.transport.memory import MemoryServer
 import bzrlib.workingtree as workingtree
 
@@ -675,3 +685,69 @@
     def test_open_containing_tree_or_branch(self):
         tree = self.make_branch_and_tree('tree')
         bzrdir.BzrDir.open_containing_tree_or_branch(self.get_url('tree'))
+
+
+class TestHTTPRedirectionLoop(object):
+    """Test redirection loop between two http servers.
+
+    This MUST be used by daughter classes that also inherit from
+    TestCaseWithTwoWebservers.
+
+    We can't inherit directly from TestCaseWithTwoWebservers or the
+    test framework will try to create an instance which cannot
+    run, its implementation being incomplete. 
+    """
+
+    # Should be defined by daughter classes to ensure redirection
+    # still use the same transport implementation
+    _qualifier = None
+
+    def create_transport_readonly_server(self):
+        return HTTPServerRedirecting()
+
+    def create_transport_secondary_server(self):
+        return HTTPServerRedirecting()
+
+    def setUp(self):
+        # Both servers redirect to each server creating a loop
+        super(TestHTTPRedirectionLoop, self).setUp()
+        # The redirections will point to the new server
+        self.new_server = self.get_readonly_server()
+        # The requests to the old server will be redirected
+        self.old_server = self.get_secondary_server()
+        # Configure the redirections
+        self.old_server.redirect_to(self.new_server.host, self.new_server.port)
+        self.new_server.redirect_to(self.old_server.host, self.old_server.port)
+
+    def _qualified_url(self, host, port):
+        return 'http+%s://%s:%s' % (self._qualifier, host, port)
+
+    def test_loop(self):
+        # Starting from either server should loop
+        old_url = self._qualified_url(self.old_server.host, 
+                                      self.old_server.port)
+        oldt = self._transport(old_url)
+        self.assertRaises(errors.NotBranchError,
+                          bzrdir.BzrDir.open_from_transport, oldt)
+        new_url = self._qualified_url(self.new_server.host, 
+                                      self.new_server.port)
+        newt = self._transport(new_url)
+        self.assertRaises(errors.NotBranchError,
+                          bzrdir.BzrDir.open_from_transport, newt)
+
+
+class TestHTTPRedirections_urllib(TestHTTPRedirectionLoop,
+                                  TestCaseWithTwoWebservers):
+    """Tests redirections for urllib implementation"""
+
+    _qualifier = 'urllib'
+    _transport = HttpTransport_urllib
+
+
+
+class TestHTTPRedirections_pycurl(TestWithTransport_pycurl,
+                                  TestHTTPRedirectionLoop,
+                                  TestCaseWithTwoWebservers):
+    """Tests redirections for pycurl implementation"""
+
+    _qualifier = 'pycurl'

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2007-02-11 16:06:13 +0000
+++ bzrlib/tests/test_http.py	2007-02-12 14:02:01 +0000
@@ -816,9 +816,15 @@
                         TestCaseWithWebserver):
     """Test the Range header in GET methods for pycurl implementation"""
 
+# FIXME: http implementations do not redirect silently anymore
+# (they do not redirect at all in fact). The mechanism is still
+# in place at the _urllib2_wrappers.Request level and tests
+# hsould be written to exercise it. For the pycurl implementation
+# the redirection have been deleted as we may deprecate pycurl.
+# -- vila 20070212
 
-class TestRedirections(object):
-    """Test redirection from the 'old' server to the 'new' server.
+class TestHTTPRedirections(object):
+    """Test redirection between http servers.
 
     This MUST be used by daughter classes that also inherit from
     TestCaseWithTwoWebservers.
@@ -830,39 +836,36 @@
 
     def create_transport_secondary_server(self):
         """Create the secondary server redirecting to the primary server"""
-        new = self.get_new_server()
-
-        return HTTPServerRedirecting(new.host, new.port)
-
-    def get_old_server(self):
-        """The redirected server"""
-        return self.get_secondary_server()
-
-    def get_new_server(self):
-        """The redirected to server"""
-        return self.get_readonly_server()
+        new = self.get_readonly_server()
+
+        redirecting = HTTPServerRedirecting()
+        redirecting.redirect_to(new.host, new.port)
+        return redirecting
 
     def setUp(self):
-        super(TestRedirections, self).setUp()
-        new = self.get_new_server()
-        old = self.get_old_server()
+        super(TestHTTPRedirections, self).setUp()
+        # The redirections will point to the new server
+        new_server = self.get_readonly_server()
+        # The requests to the old server will be redirected
+        old_server = self.get_secondary_server()
 
         self.build_tree_contents([('a', '0123456789')],)
 
-        self.new_transport = self._transport(new.get_url())
-        self.old_transport = self._transport(old.get_url())
+        self.old_transport = self._transport(old_server.get_url())
 
     def test_redirected(self):
-        old = self.old_transport
-        self.assertRaises(errors.RedirectRequested, old.get, 'a')
-
-    def test_redirection_loop(self):
-        pass
-
-
-class TestRedirections_urllib(TestRedirections,
-                              TestCaseWithTwoWebservers):
+        self.assertRaises(errors.RedirectRequested, self.old_transport.get, 'a')
+
+
+class TestHTTPRedirections_urllib(TestHTTPRedirections,
+                                  TestCaseWithTwoWebservers):
     """Tests redirections for urllib implementation"""
 
     _transport = HttpTransport_urllib
 
+
+
+class TestHTTPRedirections_pycurl(TestWithTransport_pycurl,
+                                  TestHTTPRedirections,
+                                  TestCaseWithTwoWebservers):
+    """Tests redirections for pycurl implementation"""

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2007-02-11 16:06:13 +0000
+++ bzrlib/transport/http/__init__.py	2007-02-12 14:02:01 +0000
@@ -231,7 +231,7 @@
 
         :param relpath: Path relative to transport base URL
         :param ranges: None to get the whole file;
-            or [(start,end)] to fetch parts of a file.
+            or [(start,end)+], a list of tuples to fetch parts of a file.
         :param tail_amount: to fetch that amount from file tail.
 
         :returns: (http_code, result_file)

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2007-02-11 16:06:13 +0000
+++ bzrlib/transport/http/_pycurl.py	2007-02-12 14:02:01 +0000
@@ -23,6 +23,14 @@
 # whether we expect a particular file will be modified after it's committed.
 # It's probably safer to just always revalidate.  mbp 20060321
 
+# TODO: Some refactoring could be done to avoid the strange idiom
+# used to capture data and headers while setting up the request
+# (and having to pass 'header' to _curl_perform to handle
+# redirections) . This could be achieved by creating a
+# specialized Curl object and returning code, headers and data
+# from _curl_perform.  Not done because we may deprecate pycurl in the
+# future -- vila 20070212
+
 import os
 from cStringIO import StringIO
 
@@ -102,16 +110,19 @@
         # don't want the body - ie just do a HEAD request
         # This means "NO BODY" not 'nobody'
         curl.setopt(pycurl.NOBODY, 1)
+        # But we need headers to handle redirections
+        header = StringIO()
+        curl.setopt(pycurl.HEADERFUNCTION, header.write)
         # In some erroneous cases, pycurl will emit text on
         # stdout if we don't catch it (see InvalidStatus tests
         # for one such occurrence).
         blackhole = StringIO()
         curl.setopt(pycurl.WRITEFUNCTION, blackhole.write)
-        self._curl_perform(curl)
+        self._curl_perform(curl, header)
         code = curl.getinfo(pycurl.HTTP_CODE)
         if code == 404: # not found
             return False
-        elif code in (200, 302): # "ok", "found"
+        elif code == 200: # "ok"
             return True
         else:
             self._raise_curl_http_error(curl)
@@ -156,7 +167,7 @@
         """Make a request for the entire file"""
         curl = self._curl
         abspath, data, header = self._setup_get_request(curl, relpath)
-        self._curl_perform(curl)
+        self._curl_perform(curl, header)
 
         code = curl.getinfo(pycurl.HTTP_CODE)
         data.seek(0)
@@ -179,8 +190,9 @@
             # Forget ranges, the server can't handle them
             return self._get_full(relpath)
 
-        self._curl_perform(curl, ['Range: bytes=%s'
-                                  % self.range_header(ranges, tail_amount)])
+        self._curl_perform(curl, header,
+                           ['Range: bytes=%s'
+                            % self.range_header(ranges, tail_amount)])
         data.seek(0)
 
         code = curl.getinfo(pycurl.HTTP_CODE)
@@ -201,7 +213,7 @@
         abspath, data, header = self._setup_request(curl, '.bzr/smart')
         # We override the Expect: header so that pycurl will send the POST
         # body immediately.
-        self._curl_perform(curl,['Expect: '])
+        self._curl_perform(curl, header, ['Expect: '])
         data.seek(0)
         code = curl.getinfo(pycurl.HTTP_CODE)
         headers = _extract_headers(header.getvalue(), abspath)
@@ -229,9 +241,8 @@
         # TODO: maybe include a summary of the pycurl version
         ua_str = 'bzr/%s (pycurl)' % (bzrlib.__version__,)
         curl.setopt(pycurl.USERAGENT, ua_str)
-        curl.setopt(pycurl.FOLLOWLOCATION, 1) # follow redirect responses
 
-    def _curl_perform(self, curl, more_headers=[]):
+    def _curl_perform(self, curl, header, more_headers=[]):
         """Perform curl operation and translate exceptions."""
         try:
             # There's no way in http/1.0 to say "must
@@ -264,6 +275,15 @@
             # jam 20060713 The code didn't use to re-raise the exception here,
             # but that seemed bogus
             raise
+        code = curl.getinfo(pycurl.HTTP_CODE)
+        if code in (301, 302, 303, 307):
+            url = curl.getinfo(pycurl.EFFECTIVE_URL)
+            headers = _extract_headers(header.getvalue(), url)
+            redirected_to = headers['Location']
+            raise errors.RedirectRequested(url,
+                                           redirected_to,
+                                           is_permament=(code == 301),
+                                           qual_proto=self._qualified_proto)
 
 
 def get_test_permutations():

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2007-02-11 16:06:13 +0000
+++ bzrlib/transport/http/_urllib.py	2007-02-12 14:02:01 +0000
@@ -179,12 +179,10 @@
         response = self._head(relpath)
 
         code = response.code
-        # FIXME: 302 MAY have been already processed by the
-        # redirection handler
-        if code in (200, 302): # "ok", "found"
+        if code == 200: # "ok",
             return True
         else:
-            assert(code == 404, 'Only 200, 404 or may be 302 are correct')
+            assert(code == 404, 'Only 200 or 404 are correct')
             return False
 
 

